"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("./observable");
class AwaitStore {
    constructor(_value) {
        this._value = _value;
        this.observable = new observable_1.default();
        this.subscribe = this.subscribe.bind(this);
        this.unsubscribe = this.unsubscribe.bind(this);
    }
    get value() {
        return this._value;
    }
    send(value) {
        this._value = value;
        this.observable.send(value);
    }
    subscribe(handler) {
        handler(this._value);
        this.observable.subscribe(handler);
        return () => this.unsubscribe(handler);
    }
    unsubscribe(handler) {
        this.observable.unsubscribe(handler);
    }
    awaitValue(val) {
        let ignore = () => undefined;
        let prms = new Promise(yes => {
            const cb = () => {
                if (this._value === val) {
                    yes();
                    this.observable.unsubscribe(cb);
                    return true;
                }
                return false;
            };
            if (this._value === val) {
                yes();
            }
            else {
                ignore = () => {
                    this.observable.unsubscribe(cb);
                };
                if (!cb()) {
                    this.observable.subscribe(cb);
                }
            }
        });
        return {
            then: prms.then.bind(prms),
            catch: prms.catch.bind(prms),
            ignore: () => ignore()
        };
    }
    /**
      * Creates Public API with subscribe and unsubscribe
      *
      * @returns {object}
      */
    getPublicApi() {
        if (this.observable.closed)
            throw new Error("Observable is closed!");
        return {
            subscribe: (callback) => this.subscribe(callback),
            unsubscribe: (callback) => this.unsubscribe(callback),
            awaitValue: (value) => this.awaitValue(value)
        };
    }
    close() {
        this.observable.close();
    }
}
exports.default = AwaitStore;
//# sourceMappingURL=awaiter.js.map